/******************************************************************************
 CImgWndSelect.c

							The Select Class
		
	SUPERCLASS = CImgWndTask
	
    Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <TBUtilities.h>
#include "CImgWndSelect.h"

/*** Global Variables ***/

extern RgnHandle	gUtilRgn;


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 ISelect

		Initialize a Select object. There is no task name for a
		selection, since selecting is not undoable.
 ******************************************************************************/

void	CSelect::ISelect(
	CImgWndPane	*aPane,
	CBitMap		*aBitMap)
{
	CImgWndTask::IImgWndTask(NOTHING, aPane, aBitMap);
}


/******************************************************************************
 KeepTracking
 ******************************************************************************/

void	CSelect::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongPt	 	currMapPt;
	LongPt	 	prevMapPt;
	LongPt		startMapPt;
	
	LongRect	currRect;
	LongRect	prevRect;
	RgnHandle	prevRgn;
	RgnHandle	dRgn;
	
										// Convert to bitmap coordinates
	itsPane->PaneToBitMapPt(currPt, &currMapPt);
	itsPane->PaneToBitMapPt(prevPt, &prevMapPt);
	itsPane->PaneToBitMapPt(startPt, &startMapPt);
	
	if (!EqualLongPt(&currMapPt, &prevMapPt)) {		
	
										/* Get current and previous			*/
										/*   selection rectangles			*/
		Pt2LongRect(&startMapPt, &currMapPt, &currRect);
		Pt2LongRect(&startMapPt, &prevMapPt, &prevRect);
		itsPane->LBitMapToPaneRect(&currRect, &currRect);
		itsPane->LBitMapToPaneRect(&prevRect, &prevRect);
	
										// Update current selection
		itsPane->selRect = currRect;	

										/* Here we do a lot of work to		*/
										/*   prevent flashing of the		*/
										/*   selection rectangle			*/
		
		prevRgn = NewRgn();				/* Construct region consisting of a	*/
		dRgn = NewRgn();				/*   one pixel thick outline of		*/
		LRectRgn(gUtilRgn, &currRect);	/*   the current selection rect		*/
		CopyRgn(gUtilRgn, dRgn);
		InsetRgn(dRgn, 1, 1);
		DiffRgn(gUtilRgn, dRgn, gUtilRgn);
		
		LRectRgn(prevRgn, &prevRect);	/* Construct region consisting of a	*/
		CopyRgn(prevRgn, dRgn);			/*   one pixel thick outline of		*/
		InsetRgn(dRgn, 1, 1);			/*   the previous selection rect	*/
		DiffRgn(prevRgn, dRgn, prevRgn);
		
										/* Get region which includes only	*/
										/*   those pixels that are NOT in	*/
										/*   both regions					*/
		XorRgn(gUtilRgn, prevRgn, gUtilRgn);
		DisposeRgn(prevRgn);
		DisposeRgn(dRgn);
		
		PenMode(patXor);				/* Paint this new region in gray in	*/
		PenPat(gray);					/*   Xor mode so pixels are flipped	*/
		PaintRgn(gUtilRgn);
	}
}


/******************************************************************************
 EndTracking
 ******************************************************************************/

void	CSelect::EndTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	KeepTracking(currPt, prevPt, startPt);
}

