/******************************************************************************
 CClearTask.c

							The ClearTask Class
		
	SUPERCLASS = CPaintTask
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include "CClearTask.h"

/*** Global Variables ***/

extern long			gSleepTime;			/* Max time between events			*/


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IClearTask

		Initialize a ClearTask object
 ******************************************************************************/

void	CClearTask::IClearTask(
	short		aNameIndex,
	CPaintPane	*aPaintPane,
	CBitMap		*aPainting)
{
	CPaintTask::IPaintTask(aNameIndex, aPaintPane, aPainting);
	
	done = FALSE;
}


/******************************************************************************
 Do {OVERRIDE}

		Clear the current selection in a PaintPane
 ******************************************************************************/

void	CClearTask::Do()
{
	LongRect	selRect;
	Rect		qdSelRect;

	if (!done) {
		done = TRUE;
		selRect = itsPaintPane->selRect;
		SaveForUndo(&selRect);
	
		itsPainting->BeginDrawing();	/* White out selected area			*/
		
		itsPaintPane->FrameToQDR( &selRect, &qdSelRect);
		EraseRect(&qdSelRect);
		
		itsPainting->EndDrawing();
		
		itsPaintPane->KillSelRect();	/* Now there is no selection		*/
	}
}


/******************************************************************************
 Undo {OVERRIDE}

		Undo clearing of a selection
 ******************************************************************************/

void	CClearTask::Undo()
{
	LongRect	bounds;
	
	inherited::Undo();
	
										/* If there's no current selection,	*/
										/*   select the area which had been	*/
										/*   cleared						*/
	if (EmptyLongRect(&itsPaintPane->selRect)) {
		itsUndoBits->GetBounds(&bounds);
		itsPaintPane->selRect = bounds;
		gSleepTime = 0;					/* Need to force idle to march ants	*/
	}
}


/******************************************************************************
 Redo {OVERRIDE}

		Redo clearing of a selection
 ******************************************************************************/

void	CClearTask::Redo()
{
	LongRect	bounds;
	
	inherited::Undo();

	itsUndoBits->GetBounds(&bounds);	/* If cleared area is selected,		*/
										/*   kill it so we don't march ants	*/
										/*   around the nuked space			*/
	if (EqualLongRect(&bounds, &itsPaintPane->selRect)) {
		itsPaintPane->KillSelRect();
	}
}