/******************************************************************************
 CDemoApp.c

		
	SUPERCLASS = CApplication
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	
	An example application for the THINK Class Library. CDemoApp
	can work with four different types of document:
	
	Document Type	Class	
	-------------   -----
	Plain text		CTextDoc		can create new, and open existing
	Styled text		CTextDoc		can create new, and open existing
	Table			CDemoTable  	can create new, doesn't save or open
									A demo of the CTable class
	Resource		CBrowseResDoc	can open existing, doesn't create or
									save. Can display certain resource
									types, e.g. DLOG.
	

 ******************************************************************************/

#include "CDemoApp.h"
#include "CStyleTEClipboard.h"
#include "Commands.h"
#include "DemoCommands.h"
#include "CTextDoc.h"
#include "CDemoTableDoc.h"
#include "CBartender.h"
#include "CBrowseResDoc.h"
#include "CNewDocDialog.h"
#include "CDLOGDialog.h"
#include "CPane.h"
#include "CPaneBorder.h"
#include "CStringTableDemo.h"
#include "CAppleEvent.h"

	// to suppress class stripping only
#include "CButton.h"
#include "CRadioControl.h"
#include "CRadioGroupPane.h"
#include "CStdPopupPane.h"
#include "CArrowPopupPane.h"
#include "CCheckBox.h"
#include "CIconPane.h"
#include "CIntegerText.h"

#define		kExtraMasters		10		// #times to call MoreMasters
#define		kRainyDayFund		45000	// total size of rainyDay memory reserve
#define		kCriticalBalance	40000	// portion of rainyDay saved for critical operations
#define		kToolboxBalance		20000	// portion of rainyDay saved for toolbox calls
										// and other operations that must not fail

#define 	MENUdialogs			1000	// dialog menu ID

#define		kDemoDialog1		1026	// resource IDs of demo dialogs
#define		kDemoDialog2		1027
#define		kDemoDialog3		1028
#define		kDemoDialog4		1029

	// global variables used in this file
	
extern CClipboard	*gClipboard;
extern CDesktop		*gDesktop;
extern CBartender	*gBartender;
extern OSType		gSignature;

/******************************************************************************
 CDemoApp
 
 	Initialize the global application object.
******************************************************************************/

CDemoApp::CDemoApp( void)
{
		// call CApplication's initialization method, indicating the
		// number of times to call MoreMasters, the size of the rainy day
		// fund memory reserve, and the critical operation and toolbox
		// balances
		
	CApplication::IApplication( kExtraMasters, kRainyDayFund,
						kCriticalBalance, kToolboxBalance);
						
		// The task names used in the undo menu item are stored
		// in STR# 130, in whatever order you define. If you want
		// text editing to display the correct name, e.g. "Undo Typing"
		// then you must tell CAbstractText where its strings are in
		// the STR# 130 resource.
		
	CAbstractText::cFirstTaskIndex = 1;	// first in the resource

}

/******************************************************************************
 MakeClipboard {OVERRIDE}
 
 	We override MakeClipboard in order to create a subclass of 
 	CClipboard, CStyleTEClipboard, that can display styled text.
******************************************************************************/

void	CDemoApp::MakeClipboard( void)
{
	CStyleTEClipboard	*styleClip = new( CStyleTEClipboard);
	
	styleClip->IStyleTEClipboard( this, TRUE);
	gClipboard = styleClip;
}

/******************************************************************************
 SetUpFileParameters {OVERRIDE}
 
 	You need to override SetUpFileParameters to specify your applications
 	file creator signature, and the types of files your application can open.
******************************************************************************/

void CDemoApp::SetUpFileParameters(void)
{
	inherited::SetUpFileParameters();

	sfNumTypes = 2;				// we open two types of files:
	sfFileTypes[0] = 'TEXT';	//	any text file,
	sfFileTypes[1] = 'rsrc';	//  and resource files. ResEdit uses
								// 'rsrc' as the file type for files it creates
								
	gSignature = 'tclD';		// The file creator for files your application
								// creates
}

/******************************************************************************
 SetupMenus {OVERRIDE}
 
 	We override SetupMenus in order to do some extra menu initialization.
******************************************************************************/

void	CDemoApp::SetUpMenus( void)
{
		// Creates the bartender, loads our MBAR, and installs the Apple menu
		
	inherited::SetUpMenus();
	
		// Turn off dimming and unchecking for the Font, size, and style menus
	
	AddResMenu( GetMHandle(MENUfont), 'FONT');

	gBartender->SetDimOption( MENUfont, dimNONE);
	gBartender->SetDimOption( MENUsize, dimNONE);
	gBartender->SetDimOption( MENUstyle, dimNONE);

	gBartender->SetUnchecking( MENUfont, TRUE);
	gBartender->SetUnchecking( MENUsize, TRUE);
	gBartender->SetUnchecking( MENUstyle, TRUE);
	
		// Turn off dimming for the Dialogs menu
		
	gBartender->SetDimOption( MENUdialogs, dimNONE);
}

/******************************************************************************
 DoCommand {OVERRIDE}
 
 	We override DoCommand to receive special commands handled by CDemoApp
******************************************************************************/

void	CDemoApp::DoCommand( long theCommand)
{
	CDirector 	*director;
	CWindow		*window;
	
	switch( theCommand)
	{						
		case cmdNewPlainText:
		case cmdNewStyleText:
			NewTextDoc( theCommand);
			break;
			
		case cmdNewTable:
			NewTableDoc();
			break;
			
		case cmdDemoDialog1:
			director = DoDemoDialog( kDemoDialog1, TRUE);
			break;
			
		case cmdDemoDialog2:
			director = DoDemoDialog( kDemoDialog2, FALSE);			
			break;
						
		case cmdDemoDialog3:
			director = DoDemoDialog( kDemoDialog3, TRUE);
			break;

		case cmdDemoDialog4:
			DoBorderDialog();
			break;
			
		case cmdDemoDialog5:
			DoStringTable();
			break;

		default:
			inherited::DoCommand( theCommand);
			break;
	}
}

/******************************************************************************
 CreateDocument
 
 	Here we display a dialog for the user to choose the type of document
 	to create, and then send ourselves a DoCommand message with the command
 	number corresponding to the selected document type.
 ******************************************************************************/

void	CDemoApp::CreateDocument( void)
{
	CNewDocDialog	*newDocDlg;
	long			theCommand;
	
	newDocDlg = new( CNewDocDialog);
	newDocDlg->INewDocDialog( this);
	theCommand = newDocDlg->SelectDocType();
	
	if (theCommand != cmdNull)
		DoCommand( theCommand);
			
	newDocDlg->Dispose();
}

/******************************************************************************
 NewTextDoc
 
 	Creates a new text document. If theCommand is cmdNewStyleText, the
 	document will support styled text, else it supports only plain text.
******************************************************************************/

void	CDemoApp::NewTextDoc( long theCommand)
{
	CTextDoc	*textDoc = NULL;
	
	/*
	 *	Setup an exception handler here so we can dispose of
	 *	the document if it fails during initialization, or
	 *	while creating a new document.
	 *  If an exception is raised within the scope of the
	 *	TRY block, control goes to the following CATCH
	 *	block, which does any required cleanup. Once the
	 *	CATCH block is done, the error is propagate to
	 *  the most recently posted TRY/CATCH block.
	 */
	
	TRY
	{
		textDoc = new( CTextDoc);
		textDoc->ITextDoc( this, TRUE, theCommand == cmdNewStyleText);
		textDoc->NewFile();
	}
	CATCH
	{	
		// free the document and propagate the error to the
		// next exception handler.
		
		ForgetObject( textDoc);
	}
	ENDTRY;
}

/******************************************************************************
 NewTableDoc
 
 	Create a new DemoTable document.
******************************************************************************/

void	CDemoApp::NewTableDoc( void)
{
	CDemoTableDoc	*tableDoc = NULL;
	
	/*
	 *	Setup an exception handler here so we can dispose of
	 *	the document if it fails during initialization, or
	 *	while creating a new document
	 */

	TRY
	{
		tableDoc = new( CDemoTableDoc);
		tableDoc->IDemoTableDoc();
		tableDoc->NewFile();
	}
	CATCH
	{
		ForgetObject( tableDoc);
	}
	ENDTRY;
}

/******************************************************************************
 OpenDocument  {OVERRIDE}

	Open an existing document. The type of document we open is depends upon 
	the file type, as given in macReply.
 ******************************************************************************/

void CDemoApp::OpenDocument( SFReply *macReply)
{
	CDocument		*newDoc = NULL;
	CTextDoc 		*textDoc = NULL;
	CBrowseResDoc 	*resDoc = NULL;
	
	/*
	 *	Setup an exception handler here so we can dispose of
	 *	the document if it fails during initialization, or
	 *	while opening the file.
	 */

	TRY
	{
		switch( macReply->fType)
		{
			case  'TEXT':
					textDoc = new( CTextDoc);
					newDoc = textDoc;
					textDoc->ITextDoc( this, TRUE, TRUE);
					textDoc->OpenFile( macReply);
					break;
					
			case 'rsrc':
					resDoc = new( CBrowseResDoc);
					newDoc = resDoc;
					resDoc->IBrowseResDoc( this, TRUE);
					resDoc->OpenFile( macReply);
					break;
		}
	}
	CATCH
	{
		ForgetObject( newDoc);
	}
	ENDTRY;
}

/******************************************************************************
 DoDemoDialog
 
 	Creates a dialog from the DLOG resource with the given ID. This method
 	is used for several of the dialogs opened from the Dialogs menu.
******************************************************************************/

CDirector *CDemoApp::DoDemoDialog( short resID, Boolean modal)
{
	CDLOGDirector	*dialog = NULL;
	long			cmd;
	
	TRY
	{
		dialog = new CDLOGDirector;	
		dialog->IDLOGDirector( resID, this);
		dialog->BeginDialog();
			
		if (modal)
		{
			cmd = dialog->DoModalDialog( cmdOK);	
			ForgetObject( dialog);
		}
	}
	CATCH
	{
		ForgetObject( dialog);
	}
	ENDTRY;
	
	return dialog; // not null if dialog is non-modal

}

/******************************************************************************
 DoBorderDialog
 
 	Creates the Borders demo dialog.
******************************************************************************/

void CDemoApp::DoBorderDialog( void)
{
	CDocument	*doc = NULL;
	CDLOGDialog	*dlog;
	CPane		*pane;
	short		paneID;
		
	TRY
	{
		doc = new CDocument;
		doc->IDocument( this, kPrintable);
		dlog = new CDLOGDialog;
		doc->itsWindow = dlog;
		dlog->IDLOGDialog( kDemoDialog4, gDesktop, doc);
		
			// there are 12 panes, with IDs 1-12, and 12 PBrd resources 128-139
		for (paneID = 1; paneID <= 12; paneID++)
		{
			pane = (CPane*) dlog->FindViewByID( paneID);
			if (pane)
				pane->SetResBorder( paneID + 127);
		}
		
			// set the main pane to the dialog panorama
		doc->itsMainPane = (CPane*) dlog->FindViewByID( kDialogPanoramaID);
		
			// size window so it fits on any screen
		dlog->ChangeSize( 480, 320);
		
			// show the window
		dlog->Select();
	}
	CATCH
	{
		ForgetObject( doc);
	}
	ENDTRY;
}

/******************************************************************************
 DoStringTable
 
 	Creates the string table demo dialog
******************************************************************************/

void CDemoApp::DoStringTable( void)
{
	CStringTableDemo *stringDlg = NULL;
		
	TRY
	{
		stringDlg = new CStringTableDemo;
		stringDlg->IStringTableDemo();
		stringDlg->BeginDialog();
	}
	CATCH
	{
		ForgetObject( stringDlg);
	}
	ENDTRY;

}

/******************************************************************************
 ForceClassReferences
 
 	This method creates dummy references to classes that we don't want
 	stripped out by the linker when the application is built. This could
 	happen if the class is only created via new_by_name and is never directly
 	referenced. CApplication automatically calls this method.
 	
******************************************************************************/


void	CDemoApp::ForceClassReferences( void)
{
	Boolean alwaysFalse = FALSE;
	CObject *dummy = NULL;
	
	if (alwaysFalse == TRUE)
	{
		member( dummy, CButton);
		member( dummy, CCheckBox);
		member( dummy, CRadioControl);
		member( dummy, CRadioGroupPane);
		member( dummy, CIconPane);
		member( dummy, CIntegerText);
		member( dummy, CStdPopupPane);
		member( dummy, CArrowPopupPane);	
	}
}


/******************************************************************************
 DoAppleEvent  {OVERRIDE}

	When the user chooses File/New, CDemoApp display a dialog for the
	user to choose the new doc type. Because of this interaction, we
	have to request user interation before handling the kAEOpenApplication
	AppleEvent.
	
******************************************************************************/

void CDemoApp::DoAppleEvent( CAppleEvent *anAppleEvent)
{
	OSErr		err;
	Boolean		passIt = TRUE;
		
	if ((anAppleEvent->GetEventClass() == kCoreEventClass) &&
		(anAppleEvent->GetEventID() == kAEOpenApplication))
	{
		err = anAppleEvent->RequestInteraction( MAXLONG);
		if (err != noErr)
			passIt = FALSE;
	}
	
	if (passIt)
		inherited::DoAppleEvent( anAppleEvent);

}
