/******************************************************************************
 CNewDocDialog.c

		A subclass of CDLOGDirector that displays the new document dialog, 
		allowing the user to choose the type of document to create.
		
	SUPERCLASS = CDLOGDirector
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CNewDocDialog.h"
#include "CApplication.h"
#include "Commands.h"
#include "DemoCommands.h"
#include "CRadioGroupPane.h"
#include "CWindow.h"

#define	kNewDocDLOGID	1025	// resource ID of the DLOG resource

enum	// dialog item IDs
{
	kRadioGroupID = 3,
	kNewPlainText = 5,
	kNewStyleText,
	kNewDemoTable
};

/******************************************************************************
 INewDocDialog
 
 	Initialize the dialog. This method also assigns help balloon resource
 	indexes for some of the panes in the window.
 	
******************************************************************************/

void CNewDocDialog::INewDocDialog( CDirectorOwner *aSupervisor)
{
	short	i;
	
	CDLOGDirector::IDLOGDirector( kNewDocDLOGID, aSupervisor);
	
	for (i = 1; i <= kNewDemoTable; i++)
		itsWindow->FindViewByID( i)->helpResIndex = i;
}

/******************************************************************************
 SelectDocType
 
 	Displays the modal dialog and returns a command number corresponding
 	to the user's selection.
 	
******************************************************************************/

long CNewDocDialog::SelectDocType( void)
{
	long 			theCommand;
	CRadioGroupPane	*radioGroup;
	
	// set initial radio group selection. Since this dialog was created
	// from a resource, we don't have a reference to the radio group pane.
	// Since we know its ID (its the same as the dialog item number), we
	// can get the reference by calling FindViewByID.
	
	radioGroup = (CRadioGroupPane*) itsWindow->FindViewByID( kRadioGroupID);
	radioGroup->SetStationID( kNewPlainText);
	
		// show the dialog
		
	BeginDialog();
	
		// run the dialog and return the final command.
		
	theCommand = DoModalDialog( cmdOK);
	
	if (theCommand == cmdOK)
	{
		switch( radioGroup->GetStationID())
		{
			case kNewPlainText:
				theCommand = cmdNewPlainText;
				break;

			case kNewStyleText:
				theCommand = cmdNewStyleText;
				break;

			case kNewDemoTable:
				theCommand = cmdNewTable;
				break;
				
			default:
				theCommand = cmdNull;
				break;
		}
	}
	else theCommand = cmdNull;
	
	return theCommand;
}