
/*
 *  sample cdev.c
 *
 *  Copyright (c) 1989 Symantec Corporation.  All rights reserved.
 *
 */

#include <cdev.h>


/*  cdev subclass  */

struct sample : cdev {

		/*  instance variables  */
		
	TEHandle		hTE;				/*  TE record  */
	
		/*  overridden methods  */

	long			Message(short, short);
	void			Init(void);
	void			Close(void);
	void			Update(void);
	void			Activate(void);
	void			Deactivate(void);
	void			ItemHit(short);
	void			Idle(void);
	void			Key(short);
	void			Cut(void);
	void			Copy(void);
	void			Paste(void);
	void			Clear(void);
};


/*  dialog item numbers  */

enum { iEdit = 1, iCount };


/*
 *  Runnable - should the cdev appear in the Control Panel?
 *
 *  This implements the "macDev" message.
 *
 */

Boolean
Runnable(void)
{
	return(true);
}


/*
 *  New - create the cdev object
 *
 */

cdev *
New(void)
{
	return(new(sample));
}


/*
 *  sample::Message - monitor cdev messages
 *
 *  Most cdev's would not override this method.  This one does so
 *  so it can provide a running display of what messages have arrived.
 *
 */

long
sample::Message(short msg, short item)
{
	short type;
	Handle text;
	Rect box;
	Str255 buf;
	long count;
	WindowPeek saveList;
	
	if (msg >= initDev && msg <= clearDev && msg != closeDev) {
		GetDItem(dp, lastItem + iCount + msg - initDev, &type, &text, &box);
		GetIText(text, buf);
		StringToNum(buf, &count);
		NumToString(count + 1, buf);
		if (msg == updateDev)
			CalcVis(dp);
		saveList = WindowList;
		WindowList = dp;
		dp->windowKind = dialogKind;
		SetIText(text, buf);
		dp->windowKind = refnum;
		WindowList = saveList;
	}
	return(inherited::Message(msg, item));
}


/*
 *  sample::Init
 *
 *  A TE record is created to allow entry of sample text.  For some
 *  reason, editText dialog items don't seem to work in cdev's, so
 *  we have to use a userItem.  (The item is listed as a statText item
 *  in the DITL, giving us a convenient place to store the initial
 *  text.  We change it to a userItem here.)
 *
 */

void
sample::Init(void)
{
	short type;
	Handle text;
	Rect box;
	Str255 buf;
	
	inherited::Init();
	GetDItem(dp, lastItem + iEdit, &type, &text, &box);
	GetIText(text, buf);
	SetDItem(dp, lastItem + iEdit, userItem, 0, &box);
	InsetRect(&box, 2, 2);
	hTE = TENew(&box, &box);
	TESetText(&buf[1], buf[0], hTE);
	TESetSelect(0, 32767, hTE);
	TEActivate(hTE);
	TEAutoView(true, hTE);
}


/*
 *  sample::Close
 *
 */

void
sample::Close(void)
{
	TEDispose(hTE);
	inherited::Close();
}


/*
 *  sample::Update
 *
 */

void
sample::Update(void)
{
	Rect box = (**hTE).viewRect;
	
	TEUpdate(&box, hTE);
	InsetRect(&box, -2, -2);
	FrameRect(&box);
}


/*
 *  sample::Activate
 *
 */

void
sample::Activate(void)
{
	TEActivate(hTE);
}


/*
 *  sample::Deactivate
 *
 */

void
sample::Deactivate(void)
{
	TEDeactivate(hTE);
}


/*
 *  sample::ItemHit
 *
 */

void
sample::ItemHit(short item)
{
	Point where;
	
	if (item == iEdit) {
		where = event->where;
		GlobalToLocal(&where);
		TEClick(where, (event->modifiers & shiftKey) ? true : false, hTE);
	}
}


/*
 *  sample::Idle
 *
 */

void
sample::Idle(void)
{
	TEIdle(hTE);
}


/*
 *  sample::Key
 *
 */

void
sample::Key(short c)
{
	TEKey(c, hTE);
}


/*
 *  sample::Cut
 *
 */

void
sample::Cut(void)
{
	if ((**hTE).selStart != (**hTE).selEnd)
		TECut(hTE);
}


/*
 *  sample::Copy
 *
 */

void
sample::Copy(void)
{
	if ((**hTE).selStart != (**hTE).selEnd)
		TECopy(hTE);
}


/*
 *  sample::Paste
 *
 */

void
sample::Paste(void)
{
	if (TEScrpLength)
		TEPaste(hTE);
}


/*
 *  sample::Clear
 *
 */

void
sample::Clear(void)
{
	TEDelete(hTE);
}
