
/*
 *  bufio.c
 *
 *  Copyright (c) 1991 Symantec Corporation.  All rights reserved.
 *
 */

#include "stdio.h"
#include "ansi_private.h"


/*
 *  __getc - fill the (empty) buffer and perform a "getc"
 *
 */

int
__getc(FILE *fp)
{
	fp->cnt = 0;
	__checkfile(fp);
	if (fp->pushed) {
		fp->pos += fp->cnt = fp->size;
		fp->ptr = (unsigned char *) fp->buf;
		fp->pushed = 0;
	}
	else {
		__buffer(fp);
		if (__read(fp))
			return(EOF);
	}
	--fp->cnt;
	return(*fp->ptr++);
}


/*
 *  __putc - perform a "putc" and flush the (full) buffer
 *
 */

int
__putc(int c, FILE *fp)
{
	if (!fp->dirty) {
		__checkfile(fp);
		if (fp->append)
			fp->pos = fp->len;
		__buffer(fp);
		if (fp->window)
			fp->cnt = 1;
		fp->dirty = 1;
		if (--fp->cnt)
			return(*fp->ptr++ = c);
	}
	*fp->ptr++ = c = (unsigned char) c;
	if (fp->window && c >= ' ' && ((char *) fp->ptr - fp->buf) < fp->size)
		fp->cnt = 1;
	else if (__flush(fp))
		return(EOF);
	return(c);
}


/*
 *  __buffer - prepare buffer for reading or writing
 *
 *  The buffer must be in the neutral state (cnt <= 0, !dirty).
 *
 *  To start reading after calling this routine, simply validate the
 *  buffer contents.
 *
 *  To start writing after calling this routine, copy one or more bytes
 *  into the buffer and set the "dirty" flag.
 *
 */

void
__buffer(FILE *fp)
{
	fp->ptr = (unsigned char *) fp->buf;
	fp->cnt = fp->size;
	if (fp->refnum > 0 && fp->size > 1)
		fp->cnt -= fp->pos % fp->size;
	fp->pos += fp->cnt;
}


/*
 *  __flush - force buffer to neutral state
 *
 *  If the buffer is dirty, it is written.
 *
 */

int
__flush(FILE *fp)
{
	int result = 0;
	
	fp->pos -= fp->cnt;
	if (fp->dirty) {
		fp->cnt = fp->ptr - (unsigned char *) fp->buf;
		fp->ptr = (unsigned char *) fp->buf;
		fp->dirty = 0;
		result = __write(fp);
	}
	__reset(fp);
	return(result);
}


/*
 *  __reset - initialize ptr/cnt
 *
 */

void
__reset(FILE *fp)
{
	fp->ptr = (unsigned char *) fp->buf + fp->size;
	fp->cnt = 0;
	fp->pushed = 0;
}
