/******************************************************************************
 CCheckBox.c

							The CheckBox Class
	
	A standard Mac check box control. A check box has a small square which
	may be clear or X'd (checked) and some text to the right of the box.
	Clicking in the box or the text toggles the state of the mark in the box.
	
	SUPERCLASS = CButton
	
	Copyright  1989 Symantec Corporation. All rights reserved.
	
	TCL 1.1 CHANGES
	[
		- implemented INewCheckBox for creation of checkboxes without 
		  a CNTL resource.	
	]

 ******************************************************************************/

#include "Global.h"
#include "CCheckBox.h"
#include "Commands.h"


/******************************************************************************
 ICheckBox

		Initialize a CheckBox object
 ******************************************************************************/

void	CCheckBox::ICheckBox(
	short			CNTLid,
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor)
{
	CButton::IButton(CNTLid, anEnclosure, aSupervisor);
}
	

/******************************************************************************
 INewCheckBox
 
 	Initialize a checkbox with dynamic parameters, rather than from
 	a CNTL resource.
******************************************************************************/
void CCheckBox::INewCheckBox( short aWidth, short aHeight, short aHEncl, short aVEncl, 
						StringPtr title, Boolean fVisible, 
						CView *anEnclosure,	CBureaucrat	*aSupervisor)
{
	CButton::INewButton( aWidth, aHeight, aHEncl, aVEncl, title, fVisible, 
					checkBoxProc, anEnclosure, aSupervisor);
}

/******************************************************************************
 DoGoodClick {OVERRIDE}

		A CheckBox responds to a good click (the mouse being pressed
		and released within the button) by flipping the setting on/off
		and possibly passing a command to its supervisor. Under most
		circumstances, there will be no such command.
 ******************************************************************************/

void	CCheckBox::DoGoodClick(
	short		whichPart)				/* Will always be inButton			*/
{
										/* Toggle value between 0 and 1		*/
	SetValue(1 - GetCtlValue(macControl));

	if (clickCmd != cmdNull)			/* Issue the appropriate command	*/
		itsSupervisor->DoCommand(clickCmd);
}
	

/******************************************************************************
 IsChecked

		Determine whether a CheckBox is checked or unchecked
 ******************************************************************************/

Boolean		CCheckBox::IsChecked()
{
	return( GetCtlValue(macControl) == BUTTON_ON );
}
