/******************************************************************************
 CPanorama.h

		Interface for the Panorama Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
	TCL 1.1 CHANGES
	[
		- To accomodate 32 bit coordinates, changed instance variables
		  bounds, position, and savePosition.
		- Changed interface for SetBounds, GetBounds, SetPosition, GetPosition,
		  GetHomePosition, ScrollTo, SetBounds, and GetBounds 
		  to accomodate 32 bit coordinates.	
		- added ClipToAperture method	  
	]
		
 ******************************************************************************/
 
#define _H_CPanorama

#include "CPane.h"						/* Interface for its superclass		*/

class CPanorama : public CPane {		/* Class Declaration				*/

public:
								/** Instance Variables **/
	LongRect	bounds;					/* Bounds defining Pane coordinates	*/
	short		hScale;					/* Pixels per horizontal unit		*/
	short		vScale;					/* Pixels per vertical unit			*/
	LongPt		position;				/* Location of frame in Panorama	*/
	LongPt		savePosition;			/* Save for later restoration		*/
	struct CScrollPane	*itsScrollPane;	/* Scroll pane for this Panorama	*/

								/** Instance Methods **/
									/** Contruction/Destruction **/
	void		IPanorama(CView *anEnclosure, CBureaucrat *aSupervisor,
							short aWidth, short aHeight,
							short aHEncl, short aVEncl,
							SizingOption aHSizing, SizingOption aVSizing);
	virtual void		IViewTemp(CView *anEnclosure, CBureaucrat *aSupervisor,
							Ptr viewData);
									
									/** Accessing **/
	virtual void		GetExtent(long *theHExtent, long *theVExtent);
	virtual void		GetFramePosition(long *theHPos, long *theVPos);
	virtual void		GetFrameSpan(short *theHSpan, short *theVSpan);
	virtual void		SetBounds(LongRect *aBounds);
	virtual void		GetBounds(LongRect *theBounds);
	virtual void		SetPosition(LongPt *aPosition);
	virtual void		GetPosition(LongPt *thePosition);
	virtual void		SetScales(short aHScale, short aVScale);
	virtual void		GetScales(short *theHScale, short *theVScale);
	virtual void		SetScrollPane(struct CScrollPane *aScrollPane);
	
	virtual void		GetHomePosition(LongPt *theHomePos);
	virtual void		GetPixelExtent(long *hExtent, long *vExtent);
		
									/** Calibrating **/
	virtual void		ResizeFrame(Rect *delta);

									/** Scrolling **/
	virtual void		Scroll( long hDelta, long vDelta, Boolean redraw);
	virtual void		ScrollTo(LongPt *aPosition, Boolean redraw);
	virtual void		ScrollToSelection(void);
	virtual Boolean		AutoScroll( LongPt *mouseLoc);
	    
    virtual void        DoKeyDown(char theChar, Byte keyCode, EventRecord *macEvent);
    
									/** Printing **/
									
	virtual void		Paginate( CPrinter *aPrinter, short pageWidth, short pageHeight);
	virtual void		AboutToPrint(short *firstPage, short *lastPage);
	virtual void		PrintPage(short pageNum, short pageWidth, short pageHeight, CPrinter *aPrinter);
	virtual void		DonePrinting(void);
	
};



typedef struct	{						/* Panorama template					*/
	PaneTemp	sPaneTemp;
	Rect		bounds;
	short		hScale;
	short		vScale;
	Point		position;
} PanoramaTemp, *PanoramaTempP;
	
         
void    Pane_EnclosureScrolled(CPane *thePane, LongPt *offset);
