/******************************************************************************
 CCharGrid.c

							The CharGrid Class
		
		A grid where each item is a character from a single font. Since
		fonts are just bitmaps, a font editor can be used to create one
		where each "character" is really a small picture. This is how a
		palette of tools is typically implemented.
	
	SUPERCLASS = CGridSelector
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CCharGrid.h"
#include "CTextEnvirons.h"
#include "TBUtilities.h"
#include "TCLUtilities.h"

typedef struct ChGdRec {				/* Resource template for a			*/
	short		rows;					/*   character grid					*/
	short		cols;
	short		boxWidth;
	short		boxHeight;
	short		hSizing;
	short		vSizing;
	short		hLoc;
	short		vLoc;
	short		commandBase;
	short		theSize;
	unsigned char	extra[];
} ChGdRec, *ChGdPtr, **ChGdHand;
	

/******************************************************************************
 ICharGrid

		Initialize a CharGrid object
 ******************************************************************************/

void	CCharGrid::ICharGrid(
	short			ChGdid,				/* ChGd resource ID					*/
	CView			*anEnclosure,		/* Enclosing view					*/
	CBureaucrat		*aSupervisor)		/* Boss in chain of command			*/
{
	ChGdHand			r;
	register ChGdPtr	p;
	TextInfoRec			aTextInfo;
	short				i;
	
	r = (ChGdHand) GetResource('ChGd', ChGdid);
	FailNILRes( r);
	HLock( (Handle) r);
	p = *r;
	
	CGridSelector::IGridSelector(anEnclosure, aSupervisor,
						p->hLoc, p->vLoc, p->hSizing, p->vSizing,
						1, p->commandBase, p->rows, p->cols,
						p->boxWidth, p->boxHeight);
						
	aTextInfo.theSize = p->theSize;
	aTextInfo.theStyle = 0;
	aTextInfo.theMode = srcOr;
	
	GetFontNumber((StringPtr) p->extra, &aTextInfo.fontNumber);

	i = *(p->extra + p->extra[0] + 1);
	theCharacters = NewHandleCanFail((long) i);
	FailNIL( theCharacters);
	BlockMove(p->extra + p->extra[0] + 2, *theCharacters, (Size) i); 
	
	HUnlock( (Handle) r);

	itsEnvironment = new(CTextEnvirons);
	((CTextEnvirons*)itsEnvironment)->SetTextInfo(&aTextInfo);
}
	

/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a character grid
 ******************************************************************************/

void	CCharGrid::Dispose()
{
	DisposHandle(theCharacters);		/* Get rid of list of characters	*/
	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 DrawItem {OVERRIDE}

		Draw the character at the specified row and column
 ******************************************************************************/

void	CCharGrid::DrawItem(
	short		theItem,
	Rect		*theBox)
{
	FontInfo	fInfo;					/* Info about the font				*/
	char		theChar;
	
	GetFontInfo(&fInfo);
	theChar = (*theCharacters)[theItem - 1];
	
	MoveTo(theBox->left + (boxWidth - CharWidth(theChar)) / 2,
		   theBox->bottom - (boxHeight - fInfo.ascent + fInfo.descent) / 2);
	DrawChar(theChar);
}