/******************************************************************************
 CSelectorMDEF.c

							The SelectorMDEF Class
		
	Abstract class for a Menu Definition Procedure which uses a selector as
	the basis for a menu. Most custom menus can be implemented by creating
	a subclass of this class, since a Selector embodies all the characteristics
	of a standard Mac menu: Display a group of choices and let the user
	pick one, using appropriate highlighting as the mouse moves over an item.
	
	SUPERCLASS = CPaneMDEF
	
	Copyright  1989 Symantec Corporation. All rights reserved.

	Change History:
		01/30/90	pb	Changed ChooseItem to use SetupQuickDraw and
						RestoreQuickDraw.

 ******************************************************************************/

#include "Global.h"
#include "CSelectorMDEF.h"
#include "CSelector.h"
#include "CDesktop.h"
#include "CBartender.h"

/*** Global Variables ***/
	
extern CDesktop		*gDesktop;			/* The visible Desktop				*/


/******************************************************************************
 ISelectorMDEF

		Initialize a SelectorMDEF object
 ******************************************************************************/

void	CSelectorMDEF::ISelectorMDEF(
	short			MDEFid,
	CPane			*aPane,
	CTearOffMenu	*aTearOffMenu)
{
	CPaneMDEF::IPaneMDEF(MDEFid, aPane, aTearOffMenu);
}
	

/******************************************************************************
 ChooseItem {OVERRIDE}

		Determine which item in the menu is hit.
 ******************************************************************************/

void	CSelectorMDEF::ChooseItem(
	MenuHandle		macMenu,
	Rect			*menuRect,
	Point			hitPt,
	short			*whichItem)
{
	short			theItem;
	
	if (PtInRect(hitPt, menuRect)) {
		SetupQuickDraw(menuRect);
						
		/* hitPt is expressed in global coordinates, but we need it	*/
		/* in pane coordinates. Take advantage of the fact that		*/
		/* SetupQuickDraw sets up the grafport so the origin is the	*/
		/* top left of the menu pane, and use the Toolbox's 		*/
		/* GlobalToLocal to convert the hitPt to pane coordinates.	*/
										
		GlobalToLocal(&hitPt);
		theItem = ((CSelector*)itsPane)->FindItem( hitPt);
		
		/* Support the new MenuChoice feature described in IM V-240. */
		/* The low-memory global MenuDisable is updated to reflect   */
		/* the current menu and item ID, even if the mouse is over a */
		/* disabled item or menu. The HiWord of MenuDisable contains */
		/* the menu ID and the LoWord the menu item number.			 */
		
		MenuDisable = (long)(**macMenu).menuID << 16 | (long)theItem;
		
		if (theItem != NOTHING  &&  MenuEnabled(macMenu)) {
										/* Choosing item from enabled menu	*/
										/* Provide visual feedback by		*/
										/*   dynamically hiliting the item	*/
			((CSelector*)itsPane)->HiliteItem(theItem, hiliteDYNAMIC);
			*whichItem = theItem;
		} else							/* No item is being chosen. Either	*/
			*whichItem = NOTHING;		/*   pointing at empty space or the	*/
										/*   menu is disable.				*/
		RestoreQuickDraw();

	} else {							/* Mouse is outside the menu		*/
										/* Return item number of zero		*/
		MenuDisable = (long)(**macMenu).menuID << 16;
		*whichItem = NOTHING;
										/* Tear off the menu if we have a	*/
										/*   TearOffMenu object and the		*/
										/*    menu is enabled (not dimmed)	*/
		if (itsTearOffMenu != NULL  &&  MenuEnabled(macMenu))
			TearOffMenu(menuRect, hitPt);
	}
}
