/******************************************************************************
 CBitMapPane.c

							The BitMapPane Class
		
	SUPERCLASS = CPanorama
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CBitMapPane.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IBitMapPane

		Initialize a BitMapPane object
 ******************************************************************************/

void	CBitMapPane::IBitMapPane(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing,
	LongRect		*aBounds,
	CBitMap			*aBitMap,
	Boolean			makePort)
{
	CPanorama::IPanorama(anEnclosure, aSupervisor,
					 aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing);
	
	bounds = *aBounds;
	position.h = bounds.left;
	position.v = bounds.top;
	
	if (aBitMap == NULL) {
		itsBitMap = new(CBitMap);
		itsBitMap->IBitMap(aBounds->right - aBounds->left,
						   aBounds->bottom - aBounds->top, makePort);
                           
        /*            
            Exit if we couldn't create the bitmap.
            Someone further up the call chain (i.e.,
            CArtClassDoc.BuildWindow) should detect
            the problem and tell gError to post an
            alert.
        */
        
        if (itsBitMap->macBitMap->baseAddr == NULL)
            return;
                           
		itsBitMap->SetBoundsOrigin(aBounds->left, aBounds->top);
	} else {
		itsBitMap = aBitMap;
	}
	
	autoRefresh = FALSE;
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a BitMapPane
 ******************************************************************************/

void	CBitMapPane::Dispose()
{
	if (itsBitMap != NULL) {
		itsBitMap->Dispose();
	}
	
	inherited::Dispose();
}


/**** A C C E S S I N G   M E T H O D S ****/


/******************************************************************************
 SetBitMap

		Specify the BitMap associated with a BitMapPane
 ******************************************************************************/

void	CBitMapPane::SetBitMap(
	CBitMap*	aBitMap)
{
	itsBitMap = aBitMap;				/* Set instance variable			*/
}


/******************************************************************************
 GetBitMap

		Return a reference to the BitMap associated with a BitMapPane
 ******************************************************************************/

CBitMap*	CBitMapPane::GetBitMap()
{
	return(itsBitMap);
}


/**** D R A W I N G   M E T H O D S ****/


/******************************************************************************
 Draw {OVERRIDE}

		Draw a BitMapPane
 ******************************************************************************/

void	CBitMapPane::Draw(
	Rect		*area)
{
	LongRect	lArea;
	
	if (itsBitMap != NULL) {
		SectRect(area, &itsBitMap->macBitMap->bounds, area);
		
		QDToFrameR( area, &lArea);
		itsBitMap->CopyFrom( &lArea, &lArea, NULL);
	}
}
