/******************************************************************************
 CTextEditTask.h

		Interface for text editing task
		
	SUPERCLASS = CTask
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#define _H_CTextEditTask

#include "CTask.h"


typedef struct		// saves information about a range of text
{
	Handle	text;
	long	start;
	long	end;
	long	selStart;
	long	selEnd;
	
}	tTextRange;

typedef enum		// specify either insert or deleted range of text
{
	kInsertedRange,
	kDeletedRange
	
} tRangeSelector;

typedef enum		// specify either the old or new clipboard
{
	kOldClip,
	kNewClip
	
} tClipSelector;

class CAbstractText;

class CTextEditTask : public CTask
{
public:

	void		ITextEditTask( CAbstractText *aTextPane, long anEditCmd, short firstTaskIndex);
	
	virtual void		Do(void);
	virtual void		DoTyping( char theChar, short keyCode, EventRecord *macEvent);
	virtual void		Undo(void);
	virtual void		Redo(void);
	virtual void		Dispose( void);
	virtual void		CancelTyping( void);
	virtual void		SelectionChanged( void);
	virtual Boolean		CanStillType( void);
	
protected:

	CAbstractText	*itsTextPane;		// Target text pane for this task
	long			editCmd;			// initiating command, cmdNull for typing
	tTextRange		inserted;			// info about the inserted text
	tTextRange		deleted;			// info about the deleted text
	Handle			originalScrap;		// contents of text scrap, before the task
	Boolean			stillTyping;		// true if typing is active
	Boolean			doText;				// TRUE if text should be undone
	Boolean			doClip;				// TRUE if clipboard should be undone
	EventRecord		typingEvent;		// event record for last keystroke
	
	virtual void		DoNormalChar( char theChar);
	virtual void		DoBackspace( void);
	virtual void		DoFwdDelete( void);
	
	virtual void		SaveRange( tRangeSelector whichRange);
	virtual void		DeleteRange( tRangeSelector whichRange);
	virtual void		RestoreRange( tRangeSelector whichRange, Boolean killData);
	virtual void		StoreToClip( tClipSelector whichClip);

};